import java.awt.*;
import java.awt.event.*; 
import java.util.ArrayList;

import javax.swing.JComboBox;
import javax.swing.JPanel;

/**
 * Trieda rozhrania medzi ovladanim a implementaciou jednotlivych automatov
 * Od tejto triedy budu dedit jednotlive automaty
 * @author = Ivan Klimko
 * @version = 2.0
 */
abstract public class AutomatPanel extends JPanel implements MouseListener, Runnable
{
	/**
	 *  farba gule 
	 */
	public Color farba;							
	
	/**
	 * pocet volani step()
	 */	
	public int krok=0;
				
	/**
	 * priznak autorestartu (loop) 
	 */
	public boolean autorestart = true;
	
	
	/**
	 * priznak signalu X
	 */
	public boolean signalX = true;
	
	/**
	 * priznak signalu Y
	 */
	public boolean signalY = true;
	
	/**
	 * pauza medzi prekreslenim
	 */
	private int timing = 300;
	
	/**
	 *  animacny thread 
	 */
	public volatile Thread timer = null;		//thread/vlakno pre animaciu
		
	/**
	 * Kontruktor
	 * @param BBA ui - referencia na triedu s interfaceom, ktora implementuje bba_stop();
	*/
	public AutomatPanel()
	{
		setBackground(Color.white);
		addMouseListener(this);
		farba=Color.blue;
//		SetDemo(0);
		InitSetDemo();
	}
	
	/**
     * 	funkcia pre ziskanie informacii o podporovanych demach
     * @param Choice volba - referencia na objekt rozhrania triedy java.awt.Choice
	 */
//	abstract public void GetDemos(JComboBox volba);
	
	/**
	 * 	funkcia pre nastavenie dema
	 * @param int demo - poradove cislo dema
	 */
	abstract public void SetDemo(int demo);
	
	/**
	 * 	funkcia pre nastavenie dema
	 * @param int demo - poradove cislo dema
	 */
	abstract public void InitSetDemo();
	
	/**
	 * funkcia pre pre priznak gule X / JCheckBox signalX
     * @param chb_signalX b
	 */
	abstract public void setX(boolean b);
	/**
	 * funkcia pre priznak gule Y / JCheckBox signalY
	 * @param chb_signalY b
	 */
	abstract public void setY(boolean b);
	
	/**
	 * 	funkcia reinicializuje automat
	 */
	abstract public void reset();
	
    /**
     * 	funkcia vymaze plochu automatu
	 */
	abstract public void clear();
	
	/**
	 * 	nosna funkcia - jeden krok automatu
	 */
	abstract public void step();
	
	/**
	 * 	funkcia doplnovanie elementov pouzivatelom - spracuje klik na bunku, volana nizsiev mouseClicked
	 * @param int x - suradnica x
	 * @param int y - suradnica y
	 */
	abstract protected void SetElement(int x,int y);
	
	/**
	 * 
	 *
	 */
	abstract public void saveData();
    
	//chytame mys
    //klikli sme mysou		
	/**
	 * 	  metoda obsluhy kliknutia mysi
	 */
	public void mouseClicked(MouseEvent e)
	  {
		e.consume();
		SetElement(e.getX(),e.getY());
		repaint();
		saveData();  //ulozi poslednym kliknutim navolene gule pre loop
	  }
	
    //neimplementovane ale potrebne	
	/**
	 * metoda obsluhy mysi - prazdna implementacia (vyzaduje ju rozhranie MouseListener ...
	 */
	public void mousePressed(MouseEvent e){}
	
	/**
	 * metoda obsluhy mysi - prazdna implementacia (vyzaduje ju rozhranie MouseListener ...
	 */
	public void mouseReleased(MouseEvent e){}
	/**
	 * metoda obsluhy mysi - prazdna implementacia (vyzaduje ju rozhranie MouseListener ...
	 */
	public void mouseEntered(MouseEvent e){}
	/**
	 * metoda obsluhy mysi - prazdna implementacia (vyzaduje ju rozhranie MouseListener ...
	 */
	public void mouseExited(MouseEvent e){}

//metody pre animacny thread
    /**
     * 	  metoda animacneho threadu
	 */
	public void start()    //vola metodu vlakna run
	{
		if (timer == null)          // ak sa casovac/vlakno je prazdny rob
	    {
			timer = new Thread(this, "Automat");
			timer.setPriority(Thread.NORM_PRIORITY);
			timer.start();
		}
    }
	
	/**
	 * 	  metoda animacneho threadu
	 */
	public void stop() 
	{
		timer = null;       
	}
	
	/**
	 * 	  metoda animacneho threadu (rozhrania Runnable)
	 */
	public void run()  // a zade run vola step, ktori sa implementuje v jednotl. automatoch
	{
        Thread me = Thread.currentThread(); //priradi vykonavajuce sa vlakno
        while (timer == me) {
            try 
			{
                Thread.currentThread().sleep(timing);  //prerusi cinnost vlakna na dany cas
            } 
			catch (InterruptedException e) 
			{
				e.printStackTrace();
			}
			step();
			
        }
    } 
	
	/**
	 * 	  zrychlenie behu animacneho threadu 
	 */
	public void runfaster()
	{
		timing = ((timing-75)>0)? timing-75:0;
	}
	
	/**
	 * 	  spomalenie behu animacneho threadu
	 */
	public void runslower()
	{
		timing+=75;
	}
}